# Copyright (c) 2000 Martin Schmuker. All rights reserved. 
# This program is free software; you can redistribute it and/or 
# modify it under the same terms as Perl itself. 
#
# $Header: /Tk/Icon/Icon.pm 5     5.09.00 10:46 Martin $


package Tk::Icon;

require 5.005;
#use strict;
#use warnings;
use vars qw($VERSION);

require DynaLoader;
require	Tk::Toplevel;

use Data::Dumper;

use base qw(DynaLoader Tk::Toplevel);

$VERSION = '0.03';

bootstrap Tk::Icon $VERSION;

*{"Tk::Toplevel::setIcon"} = sub {
	my $w = shift;
	
#	print Dumper(\@_);
	if ($w->viewable) {
		&Tk::Icon::setIcon($w, @_);
	} else {
#		print "Window isn't viewable. Wait 10ms...\n";
		$w->after(10, [sub {$w->setIcon(@_)}, @_]);
	}
};

#*{"Tk::Toplevel::setIcon"} = sub {
sub setIcon {
	my $w=shift;
	#print Dumper($w);
	
	# Check arguments
	die "No Window-reference supplied" unless (ref $w);
	die "Odd number of Arguments" if (@_ % 2);

	my %args = (@_);
#	print Dumper(\%args);

	die "No '-file'-Parameter supplied" unless (exists $args{-file});
	die "The icon '$args{-file}' doesn't exist" unless (-e $args{-file});

	if (exists $w->{'__icon'} && exists $w->{'__icon'}{'HICON_small'}) {
		$w->DestroyIcon($w->{'__icon'}{'HICON_small'});
		$w->DestroyIcon($w->{'__icon'}{'HICON_big'}) if ($w->{'__icon'}{'HICON_big'});
	} else {
		my $hwnd = hex($w->frame);
		$w->{__icon} = {'HWND' => $hwnd};
	}
	
	my $handle_big;
	my $handle_small = $w->ModifyIcon(\$w->{'__icon'}{'HWND'}, $args{-file}, $handle_big);
	return unless ($handle_small);
	$w->{'__icon'}{'HICON_small'} = $handle_small;
	$w->{'__icon'}{'HICON_big'} = $handle_big;
	$w->{'__icon'}{'file'} = $args{-file};
	#printf("HICON='0x%x'\n", $$handle);
}
#};

1;
__END__

=head1 NAME

Tk::Icon - Change Icon of Tk-Windows under Win32

=head1 SYNOPSIS

  use Tk;
  use Tk::Icon;
  
  my $mw = MainWindow->new;
  $mw->setIcon(-file => 'usa.ico');
  
  MainLoop;

=head1 DESCRIPTION

Tk::Icon imports the setIcon-function into the Tk::Toplevel-Package
and provides to change the Icon of a Toplevel-window under Win32.

At the moment, there is no possibility to manipulate the Icon 
using configure/cget or to destroy it. See L</TODO>.

I've tested it under Windos 98SE and Windows 2000 using Perl 5.6.0
and Tk 800.022 only, but it should work under 
all Win32-compatible operating systems because it uses documented Win32-API-Calls. 


=head1 METHODS

At the moment, there is only one method supplied:

=over 4

=item B<setIcon(>-file => I<filename>B<)>

=over 4

=item B<-file>

Specififes the Icon to display in the window's title bar. 
The ICO-File must supply an 16x16-Image that will be displayed in the 
upper left corner of the window. You can additionally supply an 32x32-Image
that will be displayed in the so-called Task-Switcher (that one that comes up
when you press Alt-Tab).



=back

When you later in the program call this method again, the previous icon will 
be destroyed and the new one will be loaded.

=back

=head1 TODO

=over 8

=item *

Rewrite Module to support configure/cget (therefore rewrite as common
Tk-Module).

=item *

Function to destroy the Icon (so that the Standard Tk-Icon will
be displayed).

=back


=head1 VERSION HISTORY

=over 8

=item * 0.03 (06 Oct 2000)

First public release.

=back


=head1 AUTHOR

Martin Schmuker, martin@schmuker.de

=cut